﻿define([
    'dojo/_base/declare',
    'dijit/_WidgetBase',
    'dijit/_TemplatedMixin',
    'dojo/text!./Templates/AddToBasketGadget.html', // the template
    'epi-cms/_ContentContextMixin', // current context (selected item)
    'dijit/_WidgetsInTemplateMixin', // required when our template contains widgets
    'dijit/form/Button', // button widgets
    'epi/i18n!./nls/localization.LWEPIServerConnector' // Enable localization using standard EPiServer language files
], function (
    declare,
    _WidgetBase,
    _TemplatedMixin,
    template,
    _ContentContextMixin,
    _WidgetsInTemplateMixin,
    button,
    translations
) {
    var $ = window.epiJQuery;

    return declare('languagewire.translation.AddToBasketGadget',
    [_WidgetBase, _TemplatedMixin, _ContentContextMixin, _WidgetsInTemplateMixin], {

        translations: translations, // translations can now be used in template

        // This is our onload for the widget
        constructor: function () {
            var that = this;
            $(window).resize(function () {
                that.onResize();
            });
           this.initializePanels();
        },

        contextChanged: function () {
            this.initializePanels();
        },

        // Returns the relative URL of the AddOn
        getRelativeBaseUrl: function () {
            var url = window.location.pathname;
            url = url.replace(/\/cms/gi, ''); // remove "/CMS"
            url = url + '/LanguageWire.Connector';
            return url;
        },

        onResize: function () {
            if (this.BasketDialog) {
                this.resizeBasketDialog();
            }
            if (this.transactionDialog) {
                this.resizeOpenTransactionDialog();
            }
            if (this.projectsDialog) {
                this.resizeProjectsDialog();
            }
            if (this.supportDialog) {
                this.resizeSupportDialog();
            }
        },

        setDialogSize: function (dialog, height, width) {
            const wHeight = $(window).height();
            const wWidth = $(window).width();

            // set height
            if (height > wHeight) {
                $(dialog).css('height', wHeight);
                $(dialog).css('overflow-y', 'scroll');
            } else {
                $(dialog).css('height', height);
                $(dialog).css('overflow-y', 'visible');
            }

            // set width
            if (width > wWidth) {
                $(dialog).css('width', wWidth);
                $(dialog).css('overflow-x', 'scroll');
            } else {
                $(dialog).css('width', width);
                $(dialog).css('overflow-x', 'visible');
            }
        },

        templateString: template,

        checkIfPageHasDescendents: function (id, language) {
            var that = this;
            $.ajax({
                type: 'POST',
                contentType: 'application/json; charset=utf-8',
                cache: false,
                async: true,
                url: that.getRelativeBaseUrl() + '/Services/ProjectWebservice.asmx/CheckPageSubitems',
                dataType: 'json',
                data: JSON.stringify({ pageId: id, language: language }),
                success: function (data) {
                    if (data.d === 'false') {
                        that.addSubBtn.setDisabled(true);
                    }
                    else {
                        that.addSubBtn.setDisabled(false);
                    }
                },
                error: function () {
                }
            });
        },

        updatePanels: function (itemsNumber, basketLanguage) {
            const ctx = this.getCurrentContext();
            var basketIsEmpty = true;
            if (itemsNumber > 0) {
                this.emptyBasket.style.display = 'none';
                this.nonEmptyBasket.style.display = 'block';
                this.basketCount.innerHTML = itemsNumber;
                this.basketLanguage.innerHTML = basketLanguage;
                this.setBasketLanguageIcon(basketLanguage);
                this.clearBasketBtn.setDisabled(false);
                this.basketBtn.setDisabled(false);
                basketIsEmpty = false;
            } else {
                this.emptyBasket.style.display = 'block';
                this.nonEmptyBasket.style.display = 'none';
                this.clearBasketBtn.setDisabled(true);
                this.basketBtn.setDisabled(true);
            }

            var addButtonDisabled = false;
            
            if ((ctx.hasTemplate === false && ctx.dataType === 'episerver.core.pagedata') // check if page is root
                || (!basketIsEmpty && basketLanguage !== ctx.language)) {
                addButtonDisabled = true;
            }

            if (this.addBtn) {
                this.addBtn.setDisabled(addButtonDisabled);
            }

            if (this.addSubBtn) {
                this.addSubBtn.setDisabled(addButtonDisabled);
            }

            if (!addButtonDisabled) {
                this.checkIfPageHasDescendents(ctx.id, ctx.language);
            }
        },

        // Update basket status panel and buttons by fetching the current basket via webservice
        initializePanels: function () {
            var that = this;
            $.ajax({
                type: 'POST',
                cache: false,
                async: true,
                data: '{}',
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                url: that.getRelativeBaseUrl() + '/Services/ProjectWebservice.asmx/GetCurrentBasket',
                success: function (data) {
                    const project = JSON.parse(data.d);
                    const ctx = that.getCurrentContext();
                    that.contextPanel.innerHTML = ctx.name;
                    that.updatePanels(project.Items.length, project.Language);
                },
                error: function () {
                }
            });
        },

        // Use web service to get icon URL for the language of the basket
        setBasketLanguageIcon: function (language) {
            var that = this;
            $.ajax({
                type: 'POST',
                contentType: 'application/json; charset=utf-8',
                cache: false,
                async: true,
                url: that.getRelativeBaseUrl() + '/Services/ProjectWebservice.asmx/GetLanguageIconUrl',
                data: "{ 'languageName': '" + language + "' }",
                dataType: 'json',

                success: function (data) {
                    const url = data.d;
                    that.basketLanguageIcon.innerHTML = '';
                    if (url) {
                        that.basketLanguageIcon.innerHTML = '<img src="' + url + '" />';
                    }
                },

                error: function () {
                }
            });
        },

        // Close ShowTransaction dialog
        closeShowTransaction: function () {
            if (this.transactionDialog) 
            {
                this.transactionDialog.dialog('close');
            }
        },

        resizeOpenTransactionDialog: function () {
            const height = 670;
            const width = 850;
            this.setDialogSize(this.transactionDialog, height, width);
        },

        // Open ShowTransaction in a modal dialog using jquery
        openTransactionDialog: function (withSubItems) {
            var that = this;
            // Build URL to ShowTransaction
            const ctx = this.getCurrentContext();
            const contentId = ctx.id;
            var url = this.getRelativeBaseUrl();
            url = url + '/ModalWindows/ShowTransaction.aspx?contentid=' + contentId + '&lang=' + ctx.language;
            var withChildren = false;
            if (withSubItems) {
                url = url + '&sub=true';
                withChildren = true;
            }

            // Display a modal dialog with ShowTransaction in an iframe (so styling is maintained)
            var height = 670;
            var width = 850;
            window.gadget = this;

            $.ajax({
                type: 'POST',
                contentType: 'application/json; charset=utf-8',
                cache: false,
                async: true,
                data: JSON.stringify({ contentId: contentId, withChildren: withChildren, language: ctx.language }),
                url: this.getRelativeBaseUrl() + '/Services/ProjectWebservice.asmx/TryAddBasketItems',
                success: function () {
                    if (!that.transactionDialog) {
                        that.transactionDialog = $('#showTransactionDialog');
                        that.transactionDialog.dialog({
                            autoOpen: false,
                            height: height,
                            width: width,
                            modal: true,
                            position: { my: 'center', at: 'center', of: window },
                            resizable: false,
                            open: function () {
                                that.resizeOpenTransactionDialog();
                            },
                            close: function (arg) {
                                if (arg.bubbles) {
                                    that.initializePanels();
                                }
                            }
                        });
                    } else {
                        that.transactionDialog.html('');
                    }
                    that.transactionDialog.html('<iframe src="' + url + '" height="' + height + 'px" width="' + width + 'px"></iframe>');
                    that.transactionDialog.dialog('open');
                },
                error: function (data) {
                    const message = JSON.parse(data.response.match(/({"d":.*})/)[1]).d;
                    alert(message);
                    console.error(message);
                }
            });
        },
        // JS implementation of String.Format()
        stringFormat: function () {
            var s = arguments[0];
            for (let i = 0; i < arguments.length - 1; i++) {
                const reg = new RegExp('\\{' + i + '\\}', 'gm');
                s = s.replace(reg, arguments[i + 1]);
            }
            return s;
        },
        // Close ShowBasket dialog
        closeShowBasket: function () {
            if (this.basketDialog) {
                this.basketDialog.dialog('close');
            }
        },
        resizeBasketDialog: function () {
            const height = 670;
            const width = 850;
            this.setDialogSize(this.basketDialog, height, width);
        },
        openBasketDialog: function () {
            // Build URL to ShowBasket
            const ctx = this.getCurrentContext();
            var url = this.getRelativeBaseUrl();
            url = url + '/ModalWindows/ShowBasketWindow/ShowBasket.aspx?contentid=' + ctx.id + '&lang=' + ctx.language;

            // Display a modal dialog with ShowBasket in an iframe (so styling is maintained)
            const height = 670;
            const width = 850;
            window.gadget = this;
            if (!this.basketDialog) {
                this.basketDialog = $('#showBasketDialog');
            } else {
                this.basketDialog.html('');
            }
            
            this.basketDialog.html('<iframe src="' + url + '" height="' + height + 'px" width="' + width + 'px"></iframe>');
            var that = this;
            this.basketDialog.dialog({
                autoOpen: false,
                height: height,
                width: width,
                modal: true,
                resizable: false,
                open: function () {
                    that.resizeBasketDialog();
                },
                close: function(arg) {
                    if (arg.bubbles) {
                        that.initializePanels();
                    }
                }
            });
            this.basketDialog.dialog('open');
        },
        // Close ShowProjects dialog
        closeShowProjects: function () {
            if (this.projectsDialog === null) return;
            this.projectsDialog.dialog('destroy');
            this.projectsDialog = null;
        },
        resizeProjectsDialog: function () {
            const height = 670;
            const width = 850;
            this.setDialogSize(this.projectsDialog, height, width);
        },
        openProjectsDialog: function () {
            // Build URL to ShowProjects
            var url = this.getRelativeBaseUrl();
            url = url + '/ModalWindows/ShowProjects/ShowProjects.aspx';
            // Display a modal dialog with ShowProjects in an iframe (so styling is maintained)
            const height = 670;
            const width = 850;
            window.gadget = this;
            this.projectsDialog = $('#showProjectsDialog');
            this.projectsDialog.html('<iframe src="' + url + '" height="' + height + 'px" width="' + width + 'px"></iframe>');
            var that = this;
            this.projectsDialog.dialog({
                autoOpen: false,
                height: height,
                width: width,
                modal: true,
                resizable: false,
                open: function () {
                    that.resizeProjectsDialog();
                }
            });
            this.projectsDialog.dialog('open');
        },
        _onAddItemClick: function () {
            this.openTransactionDialog();
        },
        _onAddWithSubItemsClick: function () {
            this.openTransactionDialog(true);
        },
        _onProjectsClick: function () {
            this.openProjectsDialog();
        },
        _onBasketClick: function () {
            this.openBasketDialog();
        },
        _onClearBasketClick: function () {
            const msg = this.stringFormat(translations.widget.confirmclear, this.basketCount.innerHTML);
            if (confirm(msg)) {
                var that = this;
                that.clearBasketBtn.setDisabled(true);
                that.basketBtn.setDisabled(true);
                $.ajax({
                    type: 'POST',
                    cache: false,
                    async: true,
                    data: '{}',
                    contentType: 'application/json; charset=utf-8',
                    dataType: 'json',
                    url: that.getRelativeBaseUrl() + '/Services/ProjectWebservice.asmx/ClearBasketSession',
                    success: function () {
                        that.updatePanels(0,'');
                    },
                    error: function () {
                        that.clearBasketBtn.setDisabled(false);
                        that.basketBtn.setDisabled(false);
                    }
                });
            }
        },
        resizeSupportDialog: function () {
            const height = 570;
            const width = 500;
            this.setDialogSize(this.basketDialog, height, width);
        },
        // Open Integrated support dialog
        openSupportDialog: function (projectId) {
            // Build URL to Support
            var url = this.getRelativeBaseUrl();
            url = url + '/ModalWindows/Support.aspx?projectId=' + projectId;

            const height = 570;
            const width = 500;
            window.gadget = this;
            this.supportDialog = $('#supportDialog');
            this.supportDialog.html('<iframe src="' + url + '" height="' + height + '" width="' + width + '"></iframe>');
            var that = this;
            this.supportDialog.dialog({
                autoOpen: false,
                height: height,
                width: width,
                modal: true,
                resizable: false,
                open: function () {
                    that.resizeSupportDialog();
                }
            });
            this.supportDialog.dialog('open');
        },
        // Close integrated support dialog
        closeSupportDialog: function () {
            if (this.supportDialog === null) return;
            this.supportDialog.dialog('destroy');
            this.supportDialog = null;
        }
    });
});